/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var mimeTypes = {
	ogg: {
		mimeType: "audio/ogg",
		testFile: "http://www.thewormlab.com/MiaowMusic/ogg/Miaow-snip-Stirring of a fool.ogg"
	},
	mp3: {
		mimeType: "audio/mpeg",
		testFile: "http://www.thewormlab.com/MiaowMusic/mp3/Miaow-snip-Stirring of a fool.mp3"
	},
	wav: {
		mimeType: "audio/x-wav",
		testFile: "http://www.thewormlab.com/MiaowMusic/wav/Miaow-snip-Stirring of a fool.wav"
	},
	au: {
		mimeType: "audio/basic",
		testFile: "http://www.thewormlab.com/MiaowMusic/au/Miaow-snip-Stirring of a fool.au"
	},
	aif: {
		mimeType: "audio/x-aiff",
		testFile: "http://www.thewormlab.com/MiaowMusic/aif/Miaow-snip-Stirring of a fool.aif"
	}
}

var audioTag = document.createElement('audio');
var audioTagSupported = !!audioTag.canPlayType;
try{
	var audioObject = new Audio("");
	var audioObjectSupported = !!audioObject.canPlayType;
}
catch(e){
	var audioObject = null;
	var audioObjectSupported = false;
}

var testAudio = function(){
	var output = document.createElement("pre");
	var mainDiv = document.getElementById("main");
	mainDiv.appendChild(output);
	var append = function(text, size, bold, end){
		end = !!end;
		bold = !!bold;
		while(text.length < size){
			text += " ";
		}
		text += end ? "\n" : " | ";
		text = document.createTextNode(text);
		if(bold){
			var bold = document.createElement("strong");
			bold.appendChild(text);
			output.appendChild(bold);
		}
		else{
			output.appendChild(text);
		}
	}
	
	var append_wrapped = function(text, limit){
		var parts = text.split(" ");
		var line = "";
		while(parts.length){
			var item = parts.shift();
			if(line.length + item.length + 1 > limit){
				output.appendChild(document.createTextNode(line + "\n"));
				line = item;
			}
			else{
				line =  line === "" ? item : line + " " + item;
			}
		}
		if(line.length){
			output.appendChild(document.createTextNode(line + "\n"));
		}
	};
	
	var size1 = 12;
	var size2 = 7;
	
	append_wrapped("Navigator: " + navigator.userAgent, 45);
	
	output.appendChild(document.createTextNode("\n"));
	append("MIME", size1, true);
	append("<audio>", size2, true);
	append("Audio()", size2, true, true);
	append("type", size1, true);
	append("tag", size2, true);
	append("object", size2, true, true);
	output.appendChild(document.createTextNode("-------------|---------|---------\n"));
	
	for(var mime in mimeTypes){
		var info = mimeTypes[mime];
		info.audioTag = audioTagSupported ? "" + audioTag.canPlayType(info.mimeType) : "(error)";
		if(info.audioTag === ""){
			info.audioTag = "no";
		}
		info.audioTagOk = info.audioTag === "maybe" || info.audioTag === "probably";
		
		info.audioObject = audioObjectSupported ? "" + audioObject.canPlayType(info.mimeType) : "error";
		if(info.audioObject === ""){
			info.audioObject = "no";
		}
		info.audioObjectOk = info.audioObject === "maybe" || info.audioObject === "probably";
		info.ok = info.audioTagOk || info.audioObjectOk;
		
		append(info.mimeType, size1, !info.ok);
		append(info.audioTag, size2, !info.audioTagOk);
		append(info.audioObject, size2, !info.audioObjectOk, true);
	}
}

window.onload = testAudio;
