/*
 * Copyright 2011-2012 Michael Mims <mims.michael@gmail.com>
 * Copyright 2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	if (window.location.href.lastIndexOf("file:///", 0) === 0){
		// We are on a local html page, hopefully home.html.
		if (Nuvola.config.homepage){
			// Redirect to the real home page.
			window.location.href = Nuvola.config.homepage;
		}
		else{
			// Show settings form
			window._Nuvola = Nuvola;
			Nuvola.amazonHomeScreen = true;
			var script = Nuvola.makeElement("script", { "src": "settings.js"});
			document.body.appendChild(script);
			document.body.removeChild(script);
		}
		return;
	}
	
	if (Nuvola.checkMp3)
		Nuvola.checkMp3();
	
	/**
	 * Creates Amazon integration binded to Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default command function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
		
		/* For debug output */
		this.name = "Amazon";
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.can_prev = null;
		this.can_next = null;
		this.update();
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		// Default state
		var state = Nuvola.STATE_NONE;
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
		var can_prev;
		var can_next;
			
		try{
			var meta = window.amznMusic.widgets.player.getCurrent().metadata;
			
			album = meta.albumName;
			album_art = meta.albumCoverImageSmall || meta.image;
			artist = meta.albumArtistName || meta.artistName;
			song = meta.title;
		}
		catch(e){
			//~ console.debug("Unable to obtain song info: " + e.message);
		}
		
		try{
			var widgets = window.amznMusic.widgets;
			state = widgets.player.isPlaying() ? Nuvola.STATE_PLAYING : Nuvola.STATE_PAUSED;
			
			if(state != Nuvola.STATE_NONE){
				can_prev = widgets.queueManager.hasPrevious();
				can_next = widgets.queueManager.hasNext();
			}
			else{
				can_prev = can_next = false;
			}
			
		}
		catch(e){
			can_prev = can_next = false;
		}

		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);
		
		if(this.can_prev !== can_prev){
			this.can_prev = can_prev;
			Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, can_prev);
		}
		if(this.can_next !== can_next){
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd){
		try{
			var player = window.amznMusic.widgets.player;
	
			switch(cmd){
			case Nuvola.ACTION_PLAY:
				if(this.state != Nuvola.STATE_PLAYING) player.playHash("play/togglePlay", "");
				break;
			case Nuvola.ACTION_PAUSE:
				player.pause();
				break;
			case Nuvola.ACTION_TOGGLE_PLAY:
				player.playHash("play/togglePlay", "");
				break;
			case Nuvola.ACTION_PREV_SONG:
				player.playHash("play/previous", "");
				break;
			case Nuvola.ACTION_NEXT_SONG:
				player.playHash("play/next", "");
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": command '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}

	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton
	
// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
