/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
if (Nuvola === window){
	if(window._Nuvola && window._Nuvola.amazonHomeScreen === true){
		// Script has been loaded from home.html
		Nuvola = window._Nuvola;
	}
	else{
		throw new Error("Nuvola Player object not found");
	}
}

if (Nuvola.Radio === undefined){
	/* Extra class not available in Nuvola JS API 2.0 */
	/**
	 * A radio option that can be added to a form.
	 */
	Nuvola.Radio = Nuvola.makeClass(Nuvola.FormWidget, {
		/**
		 * Creates a new radio option
		 * 
		 * @param group		group this radio option belongs to
		 * @param value		value of this option
		 * @param label		label shown after the entry
		 * @param checked	a state of the checkbox
		 * @param callback	a function called when the radio is modified
		 */
		$init: function(group, value, label, checked, callback){
			this.element = Nuvola.makeElement("tr");
			var cell = Nuvola.makeElement("td", {colspan: 2});
			this.input = Nuvola.makeElement("input", {
				type: "radio",
				"name": "radio-group-" + group,
				"value": value
			});
			if(checked){
				this.input.setAttribute("checked", true);
			}
			this.input.id = Nuvola.Form.nextUid();
			this.input.onchange = callback;
			cell.appendChild(this.input);
			cell.appendChild(Nuvola.makeElement("label", {"for": this.input.id}, label));
			this.element.appendChild(cell);
		}
	});
}

// Alias for gettext function, returns translated string
var _ = Nuvola.gettext;

var home_page_template = "http://www.amazon.%s/gp/dmusic/mp3/player";
var variants = [
	["de", _("Germany")],
	["fr", _("France")],
	["co.uk", _("United Kingdom")],
	["com", _("United States")]
];

var form = Nuvola.getSettingsForm();
form.reloadAction = "home";
var legend = new Nuvola.FormBox();
legend.add(Nuvola.makeText(_("Choose preferred national variant of Amazon Cloud Player.")));
form.append(legend);
if (Nuvola.amazonHomeScreen === true){
	legend = new Nuvola.FormBox();
	legend.add(Nuvola.makeText(_("This setting can be changed in the service preferences.")));
	form.append(legend);
}

var group = "variants";
for(var i = 0; i < variants.length; i++){
	var variant = variants[i];
	var value = home_page_template.replace("%s", variant[0]);
	form.append(new Nuvola.Radio(group, value, variant[1],
		Nuvola.config.homepage === value,
		function(){
			Nuvola.config.homepage = this.value;
			Nuvola.saveConfig();
			if(Nuvola.amazonHomeScreen === true){
				window.location.href = this.value;
			}
			else{
				form.showReloadNotice();
			}
		}
	));
}

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
