/*
 * Copyright 2011-2013 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola)
{
	if (Nuvola.checkMp3)
		Nuvola.checkMp3() || Nuvola.checkFlash();
	var SECTION_UNKNOWN = 0;
	var SECTION_WEEKLY = 1;
	var SECTION_DISCOVER = 2;
	var SECTION_ALBUM_VIEW = 3;
	var SECTION_TRACK_VIEW = 4;
	
	/**
	 * Creates integration binded to Nuvola JS API
	 */
	var Integration = function()
	{
		/* Overwrite default commnad function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
		
		/* For debug output */
		this.name = "Bandcamp";
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.can_thumbs_up = null;
		this.can_thumbs_down = null;
		this.can_prev = null;
		this.can_next = null;
		this.weekly = false;
		this.discover = false;
		this.section = SECTION_UNKNOWN;
		// Schedule update
		setTimeout(Nuvola.bind(this, this.update), 1000);
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function()
	{
		// Default values
		var state = Nuvola.STATE_NONE;
		var can_prev = false;
		var can_next = false;
		var can_thumbs_up = false;
		var can_thumbs_down = false;
		var album_art = null;
		var song = null;
		var artist = null;
		var album = null;
		var weeklyState = Nuvola.STATE_NONE;
		var discoverState = Nuvola.STATE_NONE;
		var albumViewState = Nuvola.STATE_NONE;
		
		var weekly = this.weekly = !! document.querySelector(".bcweekly-track-item-large");
		var discover = this.discover = !! document.querySelector("#discover_detail");
		
		if (weekly)
		{
			var elm = document.querySelector("#bcweekly.playing");
			var elm2 = document.querySelector(".play-btn .icon");
			var weeklyState = elm ? Nuvola.STATE_PLAYING : (elm2 ? Nuvola.STATE_PAUSED: Nuvola.STATE_NONE);
			if (elm)
				this.section = SECTION_WEEKLY;
		}
		
		if (discover)
		{
			var play_button = document.querySelector("#discover_detail .playbutton");
			var play_button_playing = document.querySelector("#discover_detail .playbutton.playing");
			discoverState = play_button_playing ? Nuvola.STATE_PLAYING : (play_button ? Nuvola.STATE_PAUSED: Nuvola.STATE_NONE);
			if (play_button_playing)
				this.section = SECTION_DISCOVER;
		}
		
		if (!weekly && !discover)
		{
			var play_button = document.querySelector("#trackInfo .playbutton");
			var play_button_playing = document.querySelector("#trackInfo .playbutton.playing");
			albumViewState = play_button_playing ? Nuvola.STATE_PLAYING : (play_button ? Nuvola.STATE_PAUSED: Nuvola.STATE_NONE);
			if (play_button && !! document.querySelector(".trackView[itemtype=\"http://schema.org/MusicAlbum\"]"))
				this.section = SECTION_ALBUM_VIEW;
			else if (play_button && !! document.querySelector(".trackView[itemtype=\"http://schema.org/MusicRecording\"]"))
				this.section = SECTION_TRACK_VIEW;
		}
		
		if (this.section === SECTION_WEEKLY)
		{
			var elm = document.querySelector(".bcweekly-track-item-large .track-name");
			if (elm)
				song = elm.innerText;
			
			var elm = document.querySelector(".bcweekly-track-item-large .album-name");
			if (elm)
				album = elm.innerText;
			
			var elm = document.querySelector(".bcweekly-track-item-large .artist-name");
			if (elm)
			{
				artist = elm.innerText;
				if (artist.substring(0, 3) == "by ")
					artist = artist.substring(3);
			}
			
			var elm = document.querySelector(".bcweekly-track-item-large .popupImage img");
			if (elm)
				album_art = elm.src;
			
			var elm = document.querySelector("#bcweekly.playing");
			var elm2 = this.getPlayButton();
			state = weeklyState;
		}
		else if (this.section === SECTION_DISCOVER)
		{
			var elm = document.querySelector("#discover_detail .track_info .title-section .title");
			if (elm)
				song = elm.innerText;
			var elm = document.querySelector("#discover_detail .itemtext a");
			if (elm)
				album = elm.innerText;
			var elm = document.querySelector("#discover_detail .itemsubtext a.detail_item_link_2");
			if (elm)
				artist = elm.innerText;
			var elm = document.querySelector("#discover img.detail_art");
			if (elm)
				album_art = elm.src;
			state = discoverState;
		}
		else if (this.section === SECTION_ALBUM_VIEW)
		{
			var elm = document.querySelector(".trackView .track_info .title");
			if (elm)
				song = elm.innerText;
			var elm = document.querySelector(".trackView .trackTitle");
			if (elm)
				album = elm.innerText;
			var elm = document.querySelector(".trackView span[itemprop=byArtist] a");
			if (elm)
				artist = elm.innerText;
			var elm = document.querySelector(".trackView img.popupImage");
			if (elm)
				album_art = elm.src;
			state = albumViewState;
			
			can_next = (elm = document.querySelector("#trackInfo .nextbutton")) && !elm.classList.contains("hiddenelem");
			can_prev = (elm = document.querySelector("#trackInfo .prevbutton")) && !elm.classList.contains("hiddenelem");
		}
		else if (this.section === SECTION_TRACK_VIEW)
		{
			var elm = document.querySelector(".trackView h2.trackTitle");
			if (elm)
				song = elm.innerText;
			var elm = document.querySelector(".trackView .albumTitle span[itemprop=name]");
			if (elm)
				album = elm.innerText;
			var elm = document.querySelector(".trackView span[itemprop=byArtist] a");
			if (elm)
				artist = elm.innerText;
			var elm = document.querySelector(".trackView img.popupImage");
			if (elm)
				album_art = elm.src;
			state = albumViewState;
			
			can_next = (elm = document.querySelector("#trackInfo .nextbutton")) && !elm.classList.contains("hiddenelem");
			can_prev = (elm = document.querySelector("#trackInfo .prevbutton")) && !elm.classList.contains("hiddenelem");
		}
		
		// Save state
		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);
		
		// Update actions
		if (this.can_prev !== can_prev)
		{
			this.can_prev = can_prev;
			Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, can_prev);
		}
		if (this.can_next !== can_next)
		{
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		if (this.can_thumbs_up !== can_thumbs_up)
		{
			this.can_thumbs_up = can_thumbs_up;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_UP, can_thumbs_up);
		}
		if (this.can_thumbs_down !== can_thumbs_down)
		{
			this.can_thumbs_down = can_thumbs_down;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_DOWN, can_thumbs_down);
		}
		
		// Schedule update
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	Integration.prototype.getPlayButton = function()
	{
		return this.weekly
		? document.querySelector(".play-btn .icon")
		: (this.discover
			? document.querySelector("#discover_detail .playbutton")
			: document.querySelector("#trackInfo .playbutton"));
	}
	
	Integration.prototype.clickOnQuery = function(query)
	{
		var elm = document.querySelector(query);
		if (elm)
			Nuvola.clickOnElement(elm);
		else
			console.log(this.name + ": Cannot click on '" + query + "'.");
	}
	
	
	/**
	 * Message handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd)
	{
		/* Respond to user actions */
		try
		{
			switch (cmd)
			{
			case Nuvola.ACTION_PLAY:
				if (this.state != Nuvola.STATE_PLAYING)
					Nuvola.clickOnElement(this.getPlayButton());
				break;
			case Nuvola.ACTION_PAUSE:
				if (this.state == Nuvola.STATE_PLAYING)
					Nuvola.clickOnElement(this.getPlayButton());
				break;
			case Nuvola.ACTION_TOGGLE_PLAY:
				Nuvola.clickOnElement(this.getPlayButton());
				break;
			case Nuvola.ACTION_PREV_SONG:
				this.clickOnQuery("#trackInfo .prevbutton");
				break;
			case Nuvola.ACTION_NEXT_SONG:
				this.clickOnQuery("#trackInfo .nextbutton");
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch (e)
		{
			// Older API expected exception to be a string.
			throw (this.name + ": " + e.message);
		}
	}
	
	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
