/**
 * Nuvola Player :: Deezer integration
 * 
 * This script uses jQuery, that is already included on Deezer website.
 * 
 * Copyright 2012 Michał Pawłowski <michal@pawlowski.be>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	if (Nuvola.checkFlash)
		Nuvola.checkFlash();
	
	var Integration = function(){
		this.name = "deezer";
		this.can_thumbs_down = null;
		this.can_prev = null;
		this.can_next = null;
		this.last_art = null;
		this.last_song = null;
		this.update();
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
	};
	
	Integration.prototype.update = function() {
		var state = Nuvola.STATE_NONE;
		var can_prev;
		var can_next;
		var can_thumbs_down;
		
		try{
			var play  = window.playercontrol.$btnPlay;
			var pause = window.playercontrol.$btnPause;
			
			if (play.is(':visible')) {
				state = Nuvola.STATE_PAUSED;
			} else if (pause.is(':visible')) {
				state = Nuvola.STATE_PLAYING;
			}
			
			if(state != Nuvola.STATE_NONE){
				can_prev = !window.playercontrol.$btnPrevious.hasClass('disabled');
				can_next = !window.playercontrol.$btnNext.hasClass('disabled');
				
				var td_btn = $('#like_track.bad-song');
				can_thumbs_down = td_btn.length>0 && td_btn.is(':visible');
			} else{
				can_prev = can_next = can_thumbs_down = false;
			}
			
		} catch(e){
			state = Nuvola.STATE_NONE;
			can_prev = can_next = can_thumbs_down = false;
		}
		
		var song   = window.playercontrol.$labelSongTitle.text();
		var artist = window.playercontrol.$labelArtName.text();  
		var last_song = song + '-' + artist;
		
		// album art - only on radio mode
		// remembered to next song change, when user changes tabs 
		// in main menu, album art shouldn't change to null
		if (this.last_song != last_song) {
			this.last_art = null;
		}
		
		this.last_song = last_song;
		
		try {
			var img = $('#thumb_0 img').first();
			
			if (img.length) {
				this.last_art = img.attr('src');
			}
		} catch (e) {}
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, null, this.last_art, state);
		
		// Update actions (previous song, next song)
		if(this.can_prev !== can_prev){
			this.can_prev = can_prev;
			Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, can_prev);
		}
		if(this.can_next !== can_next){
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		if(this.can_thumbs_down !== can_thumbs_down){
			this.can_thumbs_down = can_thumbs_down;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_DOWN, can_thumbs_down);
		}
		
		setTimeout(Nuvola.bind(this, this.update), 1000);
	}
	
	Integration.prototype.messageHandler = function(message) {
		try {
			switch(message){
				case Nuvola.ACTION_TOGGLE_PLAY:
					if (window.playercontrol.$btnPlay.is(':visible')) {
						window.playercontrol.doAction('play');
					} else {
						window.playercontrol.doAction('pause');
					}
					break;
				case Nuvola.ACTION_PLAY:
					window.playercontrol.doAction('play');
					break;
				case Nuvola.ACTION_PAUSE:
					window.playercontrol.doAction('pause');
					break;
				case Nuvola.ACTION_PREV_SONG:
					window.playercontrol.doAction('prev');
					break;
				case Nuvola.ACTION_NEXT_SONG:
					window.playercontrol.doAction('next');
					break;
				case Nuvola.ACTION_THUMBS_DOWN:
					window.playercontrol.doAction('smartNotLike');
					break;
				default:
					// Other commands are not supported
					throw {"message": "Not supported."};
			}
		} catch(e){
			throw (this.name + ": " + e.message);
		}
	}
	
	Nuvola.integration = new Integration(); // Singleton
})(this);
