/*
 * Copyright 2011-2012 Janez Troha <janez.troha@gmail.com>
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 * Copyright 2013 Michael J Coyne <mikeycgto@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola, $j){
	if (!$j) return alert("Unable to find window.jQuery");
	
	if (Nuvola.checkMp3)
		Nuvola.checkMp3();
	
	function get_play_button(){
		var play_button = $j('#player_play_button');

		if (play_button.length == 0) play_button = $j('#play_page span');
		if (play_button.length == 0) play_button = null;

		return play_button;
	}

	function get_favorite_button(){
		return $('#like_button');
	}
	
	/**
	 * Creates 8tracks integration binded to Nuvola JS API
	 * @param Nuvola Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);

		// We can never go back in 8tracks
		Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, false);

		this.name = "8tracks";
		this.state = Nuvola.STATE_NONE;

		this.update();
	};

	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		var meta = $('#player_box');
		var mix  = $('#mix_player_details');

		var song   = meta.find('.t').text();
		var artist = meta.find('.a').text();
		var album  = meta.find('.album .detail').text();
		var ab_art = mix.find('img').attr('src') || null;

		if (ab_art)
			ab_art = ab_art.replace(/w=\d+&h=\d+/, 'w=500&h=500');

		var play_button = $j('#player_play_button');
		var pause_button = $j('#player_pause_button');
		
		if (play_button.css('display') != "none"){
			this.state = Nuvola.STATE_PAUSED;
		} else if (pause_button.css('display') != "none"){
			this.state = Nuvola.STATE_PLAYING;
		} else {
			this.state = Nuvola.STATE_NONE;
		}

		var can_fav  = !!get_favorite_button().length;
		var can_next = true;

		// Update song information
		Nuvola.updateSong(song, artist, album, ab_art, this.state);

		// Update actions
		Nuvola.updateAction(Nuvola.ACTION_FAVORITE, can_fav);
		Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);

		// Callback in 500ms
		setTimeout(Nuvola.bind(this, this.update), 500);
	}

	/**
	 * Message Handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd){
		switch(cmd){
			case Nuvola.ACTION_PAUSE:
				if(this.state == Nuvola.STATE_PLAYING)
					$j('#player_play_button').trigger('click');
				break;

			case Nuvola.ACTION_PLAY:
				if(this.state != Nuvola.STATE_PLAYING)
					$j('#player_play_button').trigger('click');
				break;

			case Nuvola.ACTION_TOGGLE_PLAY:
				get_play_button().trigger('click');
				break;

			case Nuvola.ACTION_NEXT_SONG:
				$j('#player_skip_button').trigger('click');
				break;

			case Nuvola.ACTION_FAVORITE:
				get_favorite_button().trigger('click');

			default:
				throw this.name + ': Action not supported';

			console.log(this.name + ": command '" + cmd + "' executed.");
		}
	};

	Nuvola.integration = new Integration();

})(this, window.jQuery);

