/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	if (Nuvola.checkFlash)
		Nuvola.checkFlash();
	
	/**
	 * Creates Google Music integration binded to Nuvola JS API
	 * @param Nuvola Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.command = Nuvola.bind(this, this.command);
		
		/* For debug output */
		this.name = "GrooveShark";
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.timeout = null;
		
		
		/* Periodically try to set the callback */
		this.timeout = setInterval(Nuvola.bind(this, this.setCallback), 100);
	};
	
	/* Set callback function for GrooveShark JS API */
	Integration.prototype.setCallback = function(){
		try{
			window.Grooveshark.setSongStatusCallback(Nuvola.bind(this, this.update));
			clearInterval(this.timeout); // done!
			this.update();
		}
		catch(e){
			// Grooveshark API is probably not finished yet
		}
	}
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(current_song){
		if(!current_song){
			try{
				current_song = window.Grooveshark.getCurrentSongStatus();
			}
			catch(e){
				console.log(e.message);
			}
		}
		if(!current_song) return;
		
		var album_art = null;
		var artist = null;
		var song = null;
		var state = Nuvola.STATE_NONE;
		var album = null;
		var can_prev;
		var can_next;
		var can_thumbs_up = false;
		var can_thumbs_down = false;
		var can_favorite = false;
		
		try{
			var info = current_song.song;
			if(info){
				song = info.songName;
				album = info.albumName;
				artist = info.artistName;
				album_art = info.artURL;
			}
			
			if(current_song.status == "playing"
			|| current_song.status == "buffering"){
				state = Nuvola.STATE_PLAYING;
			}
			else if(current_song.status == "paused" || info){
				state = Nuvola.STATE_PAUSED;
			}
			else{
				state = Nuvola.STATE_NONE;
			}
		}
		catch(e){
			
		}
		
		// FIXME: check it!
		can_prev = can_next = true;		
		
		this.state = state;
		Nuvola.dataChanged(
			song, artist, album, album_art,
			state, can_prev, can_next,
			can_thumbs_up, can_thumbs_down, can_favorite);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.command = function(cmd){
		try{
			// Grooveshark provides good JavaScript API
			// See http://developers.grooveshark.com/docs/js_api/
			switch(cmd){
			case Nuvola.CMD_TOGGLE:
				if(this.state != Nuvola.STATE_PLAYING){
					window.Grooveshark.play();
				}
				else{
					window.Grooveshark.togglePlayPause();
				}
				break;
			case Nuvola.CMD_PREV_SONG:
				window.Grooveshark.previous();
				break;
			case Nuvola.CMD_PLAY:
				window.Grooveshark.play();
				break;
			case Nuvola.CMD_PAUSE:
				window.Grooveshark.pause();
				break;
			case Nuvola.CMD_NEXT_SONG:
				window.Grooveshark.next();
				break;
			case Nuvola.CMD_THUMBS_UP:
				// FIXME: Does it work? Maybe only for logged-in user?
				window.Grooveshark.voteCurrentSong(1);
				break;
			case Nuvola.CMD_THUMBS_DOWN:
				// FIXME: Does it work? Maybe only for logged-in user?
				window.Grooveshark.voteCurrentSong(-1);
				break;
			case Nuvola.CMD_FAVORITE:
				// FIXME: Does it work? Maybe only for logged-in user?
				window.Grooveshark.favoriteCurrentSong();
				break;
			default:
				// Other actions are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string
			throw (this.name + ": " + e.message);
		}
	}
	
	Nuvola.integration = new Integration(); // Singleton ;-)
})(window._Nuvola);
