/*
 * Copyright 2011-2012 Janez Troha <janez.troha@gmail.com>
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	if (Nuvola.checkFlash)
		Nuvola.checkFlash();
	
	/**
	 * Creates Hype Machine integration binded to Nuvola JS API
	 * @param Nuvola Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.command = Nuvola.bind(this, this.command);
		
		/* For debug output */
		this.name = "Hype Machine";
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.update();
		
		// Periodically update state
		this.timeout = setInterval(Nuvola.bind(this, this.update), 500);
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		var current_song = null;
		try{
			current_song = window.now_playing;
		}
		catch(e){
			console.log(e.message);
		}
		
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
		var state = Nuvola.STATE_PAUSED;
		var can_prev = false;
		var can_next = false;
		var can_thumbs_up = false;
		var can_thumbs_down = false;
		var can_favorite = false; // FIXME: There is a "favorite button" for registered users
		can_prev = can_next = true;
		try{
			if(current_song){
				if(current_song.length == 1){
					song = current_song[0].innerText;
				}
				else{
					artist = current_song[0].innerText;
					song = current_song[2].innerText;
				}
			}
		 }
		 catch(e){
			 console.log(e.message);
		 }
		
		try{
			if(window.playerStatus == "PLAYING"){
				state = Nuvola.STATE_PLAYING;
			}
			else{
				state = Nuvola.STATE_PAUSED;
			}
		}
		catch(e){
		}
		
		try{
			album_art = jQuery(".active-playing-green").find("p span")[0].style.background.replace("url(", "").replace(")", "");
		}
		catch(e){
		}
		
		this.state = state;
		Nuvola.dataChanged(
			song, artist, album, album_art,
			state, can_prev, can_next,
			can_thumbs_up, can_thumbs_down, can_favorite);
		
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.command = function(cmd){
		try{
			switch(cmd){
			case Nuvola.CMD_PAUSE:
				if(this.state == Nuvola.STATE_PLAYING)
					window.togglePlaySimple();
				break;
			case Nuvola.CMD_PLAY:
				if(this.state != Nuvola.STATE_PLAYING)
					window.togglePlaySimple();
				break;
			case Nuvola.CMD_TOGGLE:
				window.togglePlaySimple();
				break;
			case Nuvola.CMD_NEXT_SONG:
				window.nextTrack();
				break;
			case Nuvola.CMD_PREV_SONG:
				window.prevTrack();
				break;
				// FIXME: There is a "favorite button" for registered users
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string
			throw (this.name + ": " + e.message);
		}
	}
	
	Nuvola.integration = new Integration(Nuvola); // Singleton ;-)
})(window._Nuvola);
