/*
 * Copyright 2011-2012 Michael Mims <mims.michael@gmail.com>
 * Copyright 2012-2014 Jiří Janoušek <janousek.jiri@gmail.com>
 * Copyright 2013-2014 Steffen Coenen <steffen@steffen-coenen.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola) {
	if (window.location.href.lastIndexOf("file:///", 0) === 0) {
		// We are on a local html page, hopefully home.html.
		if (Nuvola.config.homepage) {
			// Redirect to the real home page.
			window.location.href = Nuvola.config.homepage;
		}
		else {
			// Show settings form
			window._Nuvola = Nuvola;
			Nuvola.lmsHomeScreen = true;
			var script = Nuvola.makeElement("script", {"src": "settings.js"});
			document.body.appendChild(script);
			document.body.removeChild(script);
		}
		return;
	}
	
	/**
	 * Creates LMS integration binded to Nuvola JS API
	 */
	var Integration = function() {
		/* Overwrite default command function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
	
		/* For debug output */
		this.name = "lms";
	
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, true);
		Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, true);
		this.update();
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function() {
		// Default state
		var state = Nuvola.STATE_NONE;
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
	
		try{
			var album_object = document.getElementById('ctrlCurrentArt').children[0];
			if (album_object.nodeName == 'IMG') {
				var album_art = album_object.src;
			}
			else{
				var album_art = album_object.children[0].src;
			}       
		}
		catch (e){
			var album_art = null;
		} 
	
		try{
			var song = document.getElementById('ctrlCurrentTitle').firstChild;
			song = song.innerText || song.textContent;
			// Kills track number if there is any one.
			if (song.indexOf(". ",0) > 0){
				song = song.split(/\.(.+)?/)[1].trim();
			}       
		}
		catch (e){
			var song = null;
		}
		var artistDiv = document.getElementById('ctrlCurrentArtist');
		try{
			var artist = artistDiv.firstChild;
			artist = artist.innerText || artist.textContent;
		}
		catch (e){
			var artist = null;
		}
		var albumDiv = document.getElementById('ctrlCurrentAlbum');
		try{
			var album = albumDiv.firstChild;
			album = album.innerText || album.textContent;
		}
		catch (e){
			var artist = null;
		} 
	
	
		var play_pause = document.getElementById('ext-gen42');
		try {
			var pp_pressed = play_pause.getAttribute("title");
			state = (pp_pressed == "Pause") ? Nuvola.STATE_PLAYING : Nuvola.STATE_PAUSED;
		}
		catch (e) {
			//state stays Nuvola.STATE_NONE
		}
		this.state = state;
	
	
	
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);
	
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd) {
	
		try {
			switch (cmd) {
				case Nuvola.ACTION_PLAY:
					if (this.state != Nuvola.STATE_PLAYING)
						document.getElementById('ext-gen42').click();
					break;
				case Nuvola.ACTION_PAUSE:
					document.getElementById('ext-gen42').click();
					break;
				case Nuvola.ACTION_TOGGLE_PLAY:
					document.getElementById('ext-gen42').click();
					break;
				case Nuvola.ACTION_PREV_SONG:
					document.getElementById('ext-gen40').click();
					break;
				case Nuvola.ACTION_NEXT_SONG:
					document.getElementById('ext-gen44').click();
					break;
				default:
					// Other commands are not supported
					throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch (e) {
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}
	
	/* Store reference */
	Nuvola.integration = new Integration(); // Singleton

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
