/*
 * Copyright 2011-2013 Jiří Janoušek <janousek.jiri@gmail.com>
 * Copyright 2013 Steffen Coenen <steffen@steffen-coenen.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

"use strict";

/* Anonymous function is used not to pollute environment */
(function(Nuvola){

if (Nuvola === window)
{
	if (window._Nuvola && window._Nuvola.lmsHomeScreen === true)
	{
		// Script has been loaded from home.html
		Nuvola = window._Nuvola;
	}
	else
	{
		throw new Error("Nuvola Player object not found");
	}
}

// Alias for gettext function, returns translated string
var _ = Nuvola.gettext;
var DEFAULT_HOST = "localhost";
var DEFAULT_PORT = "9000";

// Settings form
var form = Nuvola.getSettingsForm();
form.table.style.width = Nuvola.lmsHomeScreen === true ? "400px" : "300px";
form.reloadAction = "home";

// Some text
var legend = new Nuvola.FormBox();
legend.add(Nuvola.makeElement("strong", null, _("Address of your Logitech Media Server")));
form.append(legend);
if (Nuvola.lmsHomeScreen === true)
{
	legend = new Nuvola.FormBox();
	legend.add(Nuvola.makeText(_("This setting can be changed in the service preferences.")));
	form.append(legend);
}

// Host entry
var hostEntry = new Nuvola.Entry(_("Host"), Nuvola.config.host || DEFAULT_HOST);
hostEntry.input.style.width = "90%";
form.append(hostEntry);

// Port entry
var portEntry = new Nuvola.Entry(_("Port"), Nuvola.config.port || DEFAULT_PORT);
portEntry.input.style.width = "90%";
form.append(portEntry);

// Row with buttons
var row = Nuvola.makeElement("tr");
form.table.appendChild(row);
var cell = Nuvola.makeElement("th", {colspan: 2});
row.appendChild(cell);

// Reset button
var button = Nuvola.makeButton(_("Use default address"), function(event)
{
	event.preventDefault();
	hostEntry.input.value = DEFAULT_HOST;
	portEntry.input.value = DEFAULT_PORT;
});
button.style.minWidth = "30%";
button.style.marginLeft = "5%";
button.style.marginRight = "5%";
cell.appendChild(button);

// Apply button
var button = Nuvola.makeButton(_("Apply"), function(event)
{
	event.preventDefault();
	var host = hostEntry.input.value;
	var port = portEntry.input.value || DEFAULT_PORT;
	if (host == "")
	{
		alert(_("Host is not specified."));
	}
	else
	{
		Nuvola.config.host = host;
		Nuvola.config.port = port;
		Nuvola.config.homepage = "http://" + host + ":" + port;
		Nuvola.saveConfig();
		
		if (Nuvola.lmsHomeScreen === true)
			window.location.href = Nuvola.config.homepage;
		else
			Nuvola.triggerAction("home");
	}
});
button.style.minWidth = "30%";
button.style.marginLeft = "5%";
button.style.marginRight = "5%";
cell.appendChild(button);

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
