/*
 * Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>
 * Copyright 2012 Adam Wolfe Gordon <awg@xvx.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	if (Nuvola.checkFlash)
		Nuvola.checkFlash();
	
	/**
	 * Creates Pandora integration binded to Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.command = Nuvola.bind(this, this.command);
		
		/* For debug output */
		this.name = "Pandora";		
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.update();
		this.timeout = setInterval(Nuvola.bind(this, this.update), 500);
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		// Default state
		var state = Nuvola.STATE_NONE;
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
		var can_next;
		var can_thumbs_up;
		var can_thumbs_down;
			
		try{
			album_art = document.getElementsByClassName("art")[1].src;
			song = document.getElementsByClassName("songTitle")[0].text;
			artist = document.getElementsByClassName("artistSummary")[0].text;
			album = document.getElementsByClassName("albumTitle")[0].text;
		}
		catch(e){
			//~ console.debug("Unable to obtain song info: " + e.message);
		}
		
		try{
			var play_button_state = document.getElementsByClassName("playButton")[0].style["display"];
			var pause_button_state = document.getElementsByClassName("pauseButton")[0].style["display"];

			if(pause_button_state == "block"){
				state = Nuvola.STATE_PLAYING;
			}
			else if(play_button_state == "block"){
				state = Nuvola.STATE_PAUSED;
			}
		}
		catch(e){
			//~ console.debug("Unable to get state info: " + e.message);
		}

		// Thumbs up, thumbs down, and next are available if we're playing or paused.
		can_next = can_thumbs_up = can_thumbs_down = (state != Nuvola.STATE_NONE);

		this.state = state;

		// Submit data to Nuvola backend
		Nuvola.dataChanged(
			song, artist, album, album_art,
			state, false, can_next,
			can_thumbs_up, can_thumbs_down, false);
	}

	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.command = function(cmd){
		try{
			var skip_button = document.getElementsByClassName("skipButton")[0];
			var up_button = document.getElementsByClassName("thumbUpButton")[0];
			var down_button = document.getElementsByClassName("thumbDownButton")[0];

			switch(cmd){
			case Nuvola.CMD_PLAY:
				Pandora.sendTunerCommand("ext_play","");
				break;
			case Nuvola.CMD_PAUSE:
				Pandora.sendTunerCommand("ext_pause","");
				break;
			case Nuvola.CMD_TOGGLE:
				if(this.state == Nuvola.STATE_PAUSED){
					Pandora.sendTunerCommand("ext_play","");
				}
				else{
					Pandora.sendTunerCommand("ext_pause","");
				}
				break;
			case Nuvola.CMD_NEXT_SONG:
				Nuvola.triggerMouseEvent(skip_button, "click");
				break;
			case Nuvola.CMD_THUMBS_UP:
				Nuvola.triggerMouseEvent(up_button, "click");
				break;
			case Nuvola.CMD_THUMBS_DOWN:
				Nuvola.triggerMouseEvent(down_button, "click");
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			//~ console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}

	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton
	
// Call anonymous function with Nuvola object as an argument
})(window._Nuvola);
